@@ -36,6 +36,8 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <yara/rules.h>
 
 
+#define LOOP_LOCAL_VARS      4
+
 #define UNDEFINED           0xFFFABADAFABADAFFLL
 #define IS_UNDEFINED(x)     ((size_t)(x) == (size_t) UNDEFINED)
 
@@ -168,9 +170,6 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 
 int yr_execute_code(
-    YR_RULES* rules,
-    YR_SCAN_CONTEXT* context,
-    int timeout,
-    time_t start_time);
+    YR_SCAN_CONTEXT* context);
 
 #endif
