@@ -27,10 +27,15 @@ ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
+#if defined(JEMALLOC)
+#include <jemalloc/jemalloc.h>
+#endif
+
 #include <string.h>
 #include <stdio.h>
 #include <ctype.h>
 
+#include <yara/globals.h>
 #include <yara/error.h>
 #include <yara/re.h>
 #include <yara/modules.h>
@@ -62,9 +67,13 @@ static struct yr_config_var
     char*    str;
   };
 
-} yr_cfgs[YR_CONFIG_MAX];
+} yr_cfgs[YR_CONFIG_LAST];
 
 
+// Global variables. See globals.h for their descriptions.
+
+int yr_canary;
+
 char yr_lowercase[256];
 char yr_altercase[256];
 
@@ -78,9 +87,9 @@ char yr_altercase[256];
 static YR_MUTEX *openssl_locks;
 
 
-static unsigned long _thread_id(void)
+static void _thread_id(CRYPTO_THREADID *id)
 {
-  return (unsigned long) yr_current_thread_id();
+  CRYPTO_THREADID_set_numeric(id, (unsigned long) yr_current_thread_id());
 }
 
 
@@ -109,6 +118,7 @@ YR_API int yr_initialize(void)
 {
   uint32_t def_stack_size = DEFAULT_STACK_SIZE;
   uint32_t def_max_strings_per_rule = DEFAULT_MAX_STRINGS_PER_RULE;
+  uint32_t def_max_match_data = DEFAULT_MAX_MATCH_DATA;
 
   int i;
 
@@ -117,6 +127,10 @@ YR_API int yr_initialize(void)
   if (init_count > 1)
     return ERROR_SUCCESS;
 
+  srand((unsigned) time(NULL));
+
+  yr_canary = rand();
+
   for (i = 0; i < 256; i++)
   {
     if (i >= 'a' && i <= 'z')
@@ -141,12 +155,12 @@ YR_API int yr_initialize(void)
   for (i = 0; i < CRYPTO_num_locks(); i++)
     yr_mutex_create(&openssl_locks[i]);
 
-  CRYPTO_set_id_callback(_thread_id);
+  CRYPTO_THREADID_set_callback(_thread_id);
   CRYPTO_set_locking_callback(_locking_function);
 
   #elif defined(HAVE_WINCRYPT_H)
 
-  if (!CryptAcquireContext(&yr_cryptprov, NULL, NULL, PROV_RSA_FULL, CRYPT_VERIFYCONTEXT)) {
+  if (!CryptAcquireContext(&yr_cryptprov, NULL, NULL, PROV_RSA_AES, CRYPT_VERIFYCONTEXT)) {
     return ERROR_INTERNAL_FATAL_ERROR;
   }
 
@@ -156,15 +170,18 @@ YR_API int yr_initialize(void)
 
   #endif
 
-  FAIL_ON_ERROR(yr_re_initialize());
   FAIL_ON_ERROR(yr_modules_initialize());
 
   // Initialize default configuration options
   FAIL_ON_ERROR(yr_set_configuration(
       YR_CONFIG_STACK_SIZE, &def_stack_size));
+
   FAIL_ON_ERROR(yr_set_configuration(
       YR_CONFIG_MAX_STRINGS_PER_RULE, &def_max_strings_per_rule));
 
+  FAIL_ON_ERROR(yr_set_configuration(
+      YR_CONFIG_MAX_MATCH_DATA, &def_max_match_data));
+
   return ERROR_SUCCESS;
 }
 
@@ -172,21 +189,19 @@ YR_API int yr_initialize(void)
 //
 // yr_finalize_thread
 //
-// Should be called by ALL threads using libyara before exiting.
-//
+// This function is deprecated, it's maintained only for backward compatibility
+// with programs that already use it. Calling yr_finalize_thread from each
+// thread using libyara is not required anymore.
 
-YR_API void yr_finalize_thread(void)
+YR_DEPRECATED_API void yr_finalize_thread(void)
 {
-  yr_re_finalize_thread();
 }
 
 
 //
 // yr_finalize
 //
-// Should be called by main thread before exiting. Main thread doesn't
-// need to explicitly call yr_finalize_thread because yr_finalize already
-// calls it.
+// Should be called by main thread before exiting.
 //
 
 YR_API int yr_finalize(void)
@@ -200,8 +215,6 @@ YR_API int yr_finalize(void)
   if (init_count == 0)
     return ERROR_INTERNAL_FATAL_ERROR;
 
-  yr_re_finalize_thread();
-
   init_count--;
 
   if (init_count > 0)
@@ -213,7 +226,7 @@ YR_API int yr_finalize(void)
     yr_mutex_destroy(&openssl_locks[i]);
 
   OPENSSL_free(openssl_locks);
-  CRYPTO_set_id_callback(NULL);
+  CRYPTO_THREADID_set_callback(NULL);
   CRYPTO_set_locking_callback(NULL);
 
   #elif defined(HAVE_WINCRYPT_H)
@@ -224,10 +237,14 @@ YR_API int yr_finalize(void)
 
   FAIL_ON_ERROR(yr_thread_storage_destroy(&yr_tidx_key));
   FAIL_ON_ERROR(yr_thread_storage_destroy(&yr_recovery_state_key));
-  FAIL_ON_ERROR(yr_re_finalize());
   FAIL_ON_ERROR(yr_modules_finalize());
   FAIL_ON_ERROR(yr_heap_free());
 
+  #if defined(JEMALLOC)
+  malloc_stats_print(NULL, NULL, NULL);
+  mallctl("prof.dump", NULL, NULL, NULL, 0);
+  #endif
+
   return ERROR_SUCCESS;
 }
 
@@ -265,18 +282,39 @@ YR_API int yr_get_tidx(void)
 }
 
 
+//
+// yr_set_configuration
+//
+// Sets a configuration option. This function receives a configuration name,
+// as defined by the YR_CONFIG_NAME enum, and a pointer to the value being
+// set. The type of the value depends on the configuration name.
+//
+// Args:
+//    YR_CONFIG_NAME  name   - Any of the values defined by the YR_CONFIG_NAME
+//                             enum. Posible values are:
+//
+//       YR_CONFIG_STACK_SIZE             data type: uint32_t
+//       YR_CONFIG_MAX_STRINGS_PER_RULE   data type: uint32_t
+//       YR_CONFIG_MAX_MATCH_DATA         data type: uint32_t
+//
+//    void *src              - Pointer to the value being set for the option.
+//
+// Returns:
+//    An error code.
+
 YR_API int yr_set_configuration(
-    YR_CONFIG_NAME cfgname,
+    YR_CONFIG_NAME name,
     void *src)
 {
   if (src == NULL)
     return ERROR_INTERNAL_FATAL_ERROR;
 
-  switch (cfgname)
+  switch (name)
   { // lump all the cases using same types together in one cascade
     case YR_CONFIG_STACK_SIZE:
     case YR_CONFIG_MAX_STRINGS_PER_RULE:
-      yr_cfgs[cfgname].ui32 = *(uint32_t*) src;
+    case YR_CONFIG_MAX_MATCH_DATA:
+      yr_cfgs[name].ui32 = *(uint32_t*) src;
       break;
 
     default:
@@ -288,17 +326,18 @@ YR_API int yr_set_configuration(
 
 
 YR_API int yr_get_configuration(
-    YR_CONFIG_NAME cfgname,
+    YR_CONFIG_NAME name,
     void *dest)
 {
   if (dest == NULL)
     return ERROR_INTERNAL_FATAL_ERROR;
 
-  switch (cfgname)
+  switch (name)
   { // lump all the cases using same types together in one cascade
     case YR_CONFIG_STACK_SIZE:
     case YR_CONFIG_MAX_STRINGS_PER_RULE:
-      *(uint32_t*) dest = yr_cfgs[cfgname].ui32;
+    case YR_CONFIG_MAX_MATCH_DATA:
+      *(uint32_t*) dest = yr_cfgs[name].ui32;
       break;
 
     default:
