@@ -49,6 +49,9 @@ YR_API int yr_rules_define_integer_variable(
 {
   YR_EXTERNAL_VARIABLE* external;
 
+  if (identifier == NULL)
+    return ERROR_INVALID_ARGUMENT;
+
   external = rules->externals_list_head;
 
   while (!EXTERNAL_VARIABLE_IS_NULL(external))
@@ -76,6 +79,9 @@ YR_API int yr_rules_define_boolean_variable(
 {
   YR_EXTERNAL_VARIABLE* external;
 
+  if (identifier == NULL)
+    return ERROR_INVALID_ARGUMENT;
+
   external = rules->externals_list_head;
 
   while (!EXTERNAL_VARIABLE_IS_NULL(external))
@@ -103,6 +109,9 @@ YR_API int yr_rules_define_float_variable(
 {
   YR_EXTERNAL_VARIABLE* external;
 
+  if (identifier == NULL)
+    return ERROR_INVALID_ARGUMENT;
+
   external = rules->externals_list_head;
 
   while (!EXTERNAL_VARIABLE_IS_NULL(external))
@@ -130,6 +139,9 @@ YR_API int yr_rules_define_string_variable(
 {
   YR_EXTERNAL_VARIABLE* external;
 
+  if (identifier == NULL || value == NULL)
+    return ERROR_INVALID_ARGUMENT;
+
   external = rules->externals_list_head;
 
   while (!EXTERNAL_VARIABLE_IS_NULL(external))
@@ -471,10 +483,16 @@ YR_API int yr_rules_get_stats(
   if (match_list_lengths == NULL)
     return ERROR_INSUFFICIENT_MEMORY;
 
+  memset(stats, 0, sizeof(YR_RULES_STATS));
+
+  yr_rules_foreach(rules, rule)
+  {
+    stats->rules++;
+    yr_rule_strings_foreach(rule, string)
+      stats->strings++;
+  }
+
   stats->ac_tables_size = rules->ac_tables_size;
-  stats->ac_matches = 0;
-  stats->rules = 0;
-  stats->strings = 0;
 
   for (i = 0; i < rules->ac_tables_size; i++)
   {
@@ -501,6 +519,9 @@ YR_API int yr_rules_get_stats(
     }
   }
 
+  if (c == 0)
+    return ERROR_SUCCESS;
+
   // sort match_list_lengths in increasing order for computing percentiles.
   qsort(match_list_lengths, c, sizeof(match_list_lengths[0]), _uint32_cmp);
 
@@ -521,13 +542,6 @@ YR_API int yr_rules_get_stats(
 
   yr_free(match_list_lengths);
 
-  yr_rules_foreach(rules, rule)
-  {
-    stats->rules++;
-    yr_rule_strings_foreach(rule, string)
-      stats->strings++;
-  }
-
   return ERROR_SUCCESS;
 }
 
