@@ -196,7 +196,9 @@ static int _yr_scan_wicompare(
   if (data_size < string_length * 2)
     return 0;
 
-  while (i < string_length && yr_lowercase[*s1] == yr_lowercase[*s2])
+  while (i < string_length &&
+         yr_lowercase[*s1] == yr_lowercase[*s2] &&
+         *(s1 + 1) == 0x00)
   {
     s1+=2;
     s2++;
@@ -872,13 +874,9 @@ int yr_scan_verify_match(
 
   #ifdef PROFILING_ENABLED
   uint64_t finish_time = yr_stopwatch_elapsed_us(&context->stopwatch);
-  #ifdef _WIN32
-  InterlockedAdd64(&string->time_cost,  finish_time - start_time);
-  InterlockedAdd64(&string->rule->time_cost,  finish_time - start_time);
-  #else
-  __sync_fetch_and_add(&string->time_cost, finish_time - start_time);
-  __sync_fetch_and_add(&string->rule->time_cost, finish_time - start_time);
-  #endif
+
+  string->rule->time_cost_per_thread[context->tidx] += (
+      finish_time - start_time);
   #endif
 
   return result;
